/*
 * Copyright (C) 2010 Yury P. Fedorchenko
 * yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class ChBoard extends Canvas implements Runnable {
  static int CellSize;
  int cursorx, cursory;
  int cursorx_old, cursory_old;
  private static final int CHAR_HEIGHT;
  static {
    Font defaultFont = Font.getDefaultFont();
    CHAR_HEIGHT = defaultFont.getHeight();
  }
  private chgame.Checkers theBoard;
  private boolean gameOver = false;
  private boolean gameDestroyed = false;
  private int width;
  private int height;
  boolean isWhite;

  public ChBoard() {
    width = getWidth();
    height = getHeight();
    ChBoard.CellSize = ((height / 8) < (width / 8)) ? (height / 8) : (width / 8);
    isWhite = true;
    theBoard = new chgame.Checkers(isWhite);
    cursorx = cursory = 0;
    cursorx_old = -1;
    cursory_old = -1;
  }

  void restart() {
    theBoard.newGame(isWhite);
    cursorx = cursory = 0;
    cursorx_old = -1;
    cursory_old = -1;
  }

  public void keyPressed(int keyCode) {
    if (theBoard.gameOver()) {
      return;
    }
    synchronized (theBoard) {
      int action = getGameAction(keyCode);
      switch (action) {
        case Canvas.LEFT:
          cursorx--;
          if (cursorx < 0) {
            cursorx = 7;
          }
          break;
        case Canvas.RIGHT:
          cursorx++;
          if (cursorx > 7) {
            cursorx = 0;
          }
          break;
        case Canvas.DOWN:
          cursory++;
          if (cursory > 7) {
            cursory = 0;
          }
          break;
        case Canvas.UP:
          cursory--;
          if (cursory < 0) {
            cursory = 7;
          }
          break;
        case Canvas.FIRE:
        case Canvas.GAME_A://for nokia touch screen phone
          if ((cursorx + cursory) % 2 == 0) {
            return;
          }
          if (cursorx_old == -1 || cursory_old == -1) {
            cursorx_old = cursorx;
            cursory_old = cursory;
          } else if (theBoard.move(cursorx_old, cursory_old, cursorx, cursory, this)) {
            cursorx_old = -1;
            cursory_old = -1;
          } else {
            cursorx_old = cursorx;
            cursory_old = cursory;
          }
          break;
        default:
          switch (keyCode) {
            case Canvas.KEY_NUM2:
              cursory--;
              if (cursory < 0) {
                cursory = 7;
              }
              break;
            case Canvas.KEY_NUM8:
              cursory++;
              if (cursory > 7) {
                cursory = 0;
              }
              break;
            case Canvas.KEY_NUM4:
              cursorx--;
              if (cursorx < 0) {
                cursorx = 7;
              }
              break;
            case Canvas.KEY_NUM6:
              cursorx++;
              if (cursorx > 7) {
                cursorx = 0;
              }
              break;
            case Canvas.KEY_NUM5:
              if ((cursorx + cursory) % 2 == 0) {
                return;
              }
              if (cursorx_old == -1 || cursory_old == -1) {
                cursorx_old = cursorx;
                cursory_old = cursory;
              } else if (theBoard.move(cursorx_old, cursory_old, cursorx, cursory, this)) {
                cursorx_old = -1;
                cursory_old = -1;
              } else {
                cursorx_old = cursorx;
                cursory_old = cursory;
              }
            default:
              return;
          }
      }
      repaint();
    }
  }

  void drawChecker(Graphics g, int i, int j, boolean white, boolean king) {
    if (white) {
      g.setColor(0, 0, 0);
      g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
      g.setColor(255, 255, 255);
      g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
      if (king) {
        g.setColor(0, 0, 0);
        g.setFont(Font.getDefaultFont());
        g.drawChar('w', (int) ((i + 0.5) * CellSize),
                (int) ((j + 0.5) * CellSize + CHAR_HEIGHT / 2), Graphics.BOTTOM | Graphics.HCENTER);
      }
    } else {
      g.setColor(255, 255, 255);
      g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
      g.setColor(0, 0, 0);
      g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
      if (king) {
        g.setColor(255, 255, 255);
        g.setFont(Font.getDefaultFont());
        g.drawChar('w', (int) ((i + 0.5) * CellSize),
                (int) ((j + 0.5) * CellSize + CHAR_HEIGHT / 2), Graphics.BOTTOM | Graphics.HCENTER);
      }
    }
  }

  public void paint(Graphics g) {
    g.setColor(200, 200, 200);
    g.fillRect(0, 0, getWidth(), getHeight());
    for (int i = 0; i < 8; ++i) {
      for (int j = 0; j < 8; ++j) {
        if ((i + j) % 2 == 0) {
          g.setColor(192, 192, 192);
          g.fillRect(i * CellSize, j * CellSize, CellSize, CellSize);
          if (i == cursorx && j == cursory) {
            g.setColor(64, 64, 64);
            g.fillRect(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8);
          }
          if (i == cursorx_old && j == cursory_old) {
            g.setColor(64, 64, 64);
            g.fillRect(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8);
          }
        } else {
          g.setColor(64, 64, 64);
          g.fillRect(i * CellSize, j * CellSize, CellSize, CellSize);
          if (i == cursorx && j == cursory) {
            g.setColor(192, 192, 192);
            g.fillRect(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8);
          }
          if (i == cursorx_old && j == cursory_old) {
            g.setColor(192, 192, 192);
            g.fillRect(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8);
          }
          switch (theBoard.item(i, j)) {
            case 1:
              if (isWhite) {
                g.setColor(0, 0, 0);
                g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
                g.setColor(255, 255, 255);
                g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
              } else {
                g.setColor(255, 255, 255);
                g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
                g.setColor(0, 0, 0);
                g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
              }
              break;
            case 2:
              drawChecker(g, i, j, isWhite, true);
              break;
            case 5:
              if (!isWhite) {
                g.setColor(0, 0, 0);
                g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
                g.setColor(255, 255, 255);
                g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
              } else {
                g.setColor(255, 255, 255);
                g.drawArc(i * CellSize + 4, j * CellSize + 4, CellSize - 8, CellSize - 8, 0, 360);
                g.setColor(0, 0, 0);
                g.fillArc(i * CellSize + 5, j * CellSize + 5, CellSize - 10, CellSize - 10, 0, 360);
              }
              break;
            case 4:
              drawChecker(g, i, j, !isWhite, true);
              break;
          }
        }
      }
      g.setFont(Font.getDefaultFont());
      g.setColor(0, 0, 0);
      if (getWidth() > getHeight()) {
        g.drawString("level:" + CheckersMain.levels[getLevel()]+" type "+((theBoard.type()==0)?"Eng":"Rus"), 8 * CellSize, 0 + CHAR_HEIGHT, Graphics.TOP | Graphics.LEFT);
        if (theBoard.gameOver()) {
          if (theBoard.winner() == 2) {
            g.drawString("You Win", 8 * CellSize, CHAR_HEIGHT + 3, Graphics.TOP | Graphics.LEFT);
          }
          if (theBoard.winner() == 1) {
            g.drawString("Phone Win", 8 * CellSize, CHAR_HEIGHT + 3, Graphics.TOP | Graphics.LEFT);
          }
          if (theBoard.winner() == 3) {
            g.drawString("Game over", 8 * CellSize, CHAR_HEIGHT + 3, Graphics.TOP | Graphics.LEFT);
          }
        }
      } else {
        g.drawString("level:" + CheckersMain.levels[getLevel()]+" type "+((theBoard.type()==0)?"Eng":"Rus"), 0, 8 * CellSize, Graphics.TOP | Graphics.LEFT);
        if (theBoard.gameOver()) {
          if (theBoard.winner() == 2) {
            g.drawString("You Win", 0, 8 * CellSize + CHAR_HEIGHT + 2, Graphics.TOP | Graphics.LEFT);
          }
          if (theBoard.winner() == 1) {
            g.drawString("Phone Win", 0, 8 * CellSize + CHAR_HEIGHT + 2, Graphics.TOP | Graphics.LEFT);
          }
          if (theBoard.winner() == 3) {
            g.drawString("Game over", 0, 8 * CellSize + CHAR_HEIGHT + 2, Graphics.TOP | Graphics.LEFT);
          }
        }
      }
    }
  }

  protected void hideNotify() {
    super.hideNotify();
  }

  public void run() {
    while (!gameDestroyed) {
      try {
        synchronized (theBoard) {
          if (gameOver) {
            repaint();
            theBoard.wait(); // Wait until user presses 'restart'
          }
          repaint();
        }
      } catch (java.lang.InterruptedException ie) {
      }
    }
  }
  int getType(){
    return theBoard.type();
  }
  void setType(int tp){
    theBoard.type(tp);
  }
  int getLevel() {
    int lv = theBoard.getSkill();
    lv -= 4;
    if (lv < 0) {
      return 0;
    } else if (lv == 0) {
      return 1;
    }
    return lv;
  }

  void setLevel(int i) {
    if (i > 1) {
      i += 4;
    }
    if (i == 1) {
      i = 4;
    }
    if (i == 0) {
      i = 2;
    }
    theBoard.setSkill(i);
  }

  public void destroyGame() {
    synchronized (theBoard) {
      gameDestroyed = true;
      theBoard.notifyAll();
    }

  }

  public void setDestroyed(boolean destroyGame) {
    this.gameDestroyed = destroyGame;
  }
}
