/*
 * Copyright (C) 2010 Yury P. Fedorchenko
 * yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Item;
import javax.microedition.midlet.MIDlet;

public class CheckersMain extends MIDlet implements CommandListener {

  private ChBoard theGame;
  private Command exitCmd = new Command("Exit", Command.EXIT, 3);
  private Command optionCmd = new Command("Options", Command.SCREEN, 2);
  private Command startCmd = new Command("Start", Command.SCREEN, 1);
  private Command restartCmd = new Command("Restart", Command.SCREEN, 1);
  private Command cancelCmd = new Command("Cancel", Command.ITEM, 1);
  private Command aboutCmd = new Command("About", Command.SCREEN, 4);
  private Command OKCmd = new Command("OK", Command.OK, 1);
  private Thread myThread;
  ChoiceGroup chgmtp;
  ChoiceGroup chlvl;
  ChoiceGroup chcpl;
  static public String[] levels = {"Beginner", "Novice", "Average", "Good", "Expert", "Master"};

  public CheckersMain() {
    theGame = new ChBoard();
    theGame.addCommand(exitCmd);
    theGame.addCommand(optionCmd);
    theGame.addCommand(startCmd);
    theGame.addCommand(aboutCmd);
    theGame.setCommandListener(this);
  }

  protected void destroyApp(boolean unconditional) {
    theGame.destroyGame();
    Display.getDisplay(this).setCurrent((Displayable) null);
  }

  protected void pauseApp() {
    theGame.destroyGame();
    try {
      myThread.join();
    } catch (InterruptedException ie) {
      //
    }
  }

  protected void startApp() {
    Display.getDisplay(this).setCurrent(theGame);
    try {
      myThread = new Thread(theGame);
      theGame.setDestroyed(false);
      myThread.start();
    } catch (Error e) {
      destroyApp(false);
      notifyDestroyed();
    }
  }

  public void commandAction(Command c, Displayable d) {
    if (c == restartCmd) {
      theGame.restart();
    } else if (c == optionCmd) {
      String[] cpls = {"White", "Black"};
      String[] cpgt = {"English", "Russian"};
      chgmtp = new ChoiceGroup("Game type", ChoiceGroup.EXCLUSIVE, cpgt, null);
      chlvl = new ChoiceGroup("Level", ChoiceGroup.POPUP, levels, null);
      chcpl = new ChoiceGroup("Player Color", ChoiceGroup.POPUP, cpls, null);
      Item[] levelItem = {chgmtp, chlvl, chcpl};
      chgmtp.setSelectedIndex(theGame.getType(), true);
      chlvl.setSelectedIndex(theGame.getLevel(), true);
      chcpl.setSelectedIndex(theGame.isWhite ? 0 : 1, true);
      final Form f = new Form("Options", levelItem);
      f.addCommand(OKCmd);
      f.addCommand(cancelCmd);
      final MIDlet m = this;
      f.setCommandListener(new CommandListener() {

        public void commandAction(Command arg0, Displayable arg1) {
          if (arg0 == OKCmd) {
            theGame.setType(chgmtp.getSelectedIndex());
            theGame.setLevel(chlvl.getSelectedIndex());
            theGame.isWhite = chcpl.getSelectedIndex() == 0;
            theGame.restart();
            Display.getDisplay(m).setCurrent(theGame);
          } else if (arg0 == cancelCmd) {
            Display.getDisplay(m).setCurrent(theGame);
          }
        }
      });
      Display.getDisplay(this).setCurrent(f);
    } else if (c == exitCmd) {
      destroyApp(false);
      notifyDestroyed();
    } else if (c == aboutCmd) {
      Item[] aboutItem = {new StringItem("Yacheckers 1.1.0 (MIPD version)", "\nby Yury P. Fedorchenko\n" +
        "This is the free software and distributed under the term of GPL\n" +
        "last versions are aviable at http://www.fedorchenko.net")};
      Form f = new Form("About", aboutItem);
      f.addCommand(OKCmd);
      final MIDlet m = this;
      f.setCommandListener(new CommandListener() {

        public void commandAction(Command arg0, Displayable arg1) {
          Display.getDisplay(m).setCurrent(theGame);
          //throw new UnsupportedOperationException("Not supported yet.");
        }
      });
      Display.getDisplay(this).setCurrent(f);
    } else if (c == startCmd) {
      theGame.removeCommand(startCmd);
      theGame.addCommand(restartCmd);
      theGame.restart();
    }
  }
}
