/*
 * Copyright (C) 2010 Yury P. Fedorchenko
 * yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

package chgame;

class IntGrave{
  int val;
  public IntGrave(int v){val=v;}
  public void set(int v){val=v;}
  public int  get(){return val;}
};
class BoolGrave{
  boolean val;
  public BoolGrave(boolean v){val=v;}
  public void set(boolean v){val=v;}
  public boolean  get(){return val;}
};

abstract class GameInternal {

  static final int NONE = 0, MAN1 = 1, KING1 = 2, FREE = 3, KING2 = 4, MAN2 = 5, FULL = 6;
  static final int UL = -6, UR = -5, DL = 5, DR = 6;

  int internal(int external) {
    int[] i = {6, 7, 8, 9, 11, 12, 13, 14, 17, 18, 19, 20, 22, 23, 24, 25,
      28, 29, 30, 31, 33, 34, 35, 36, 39, 40, 41, 42, 44, 45, 46, 47};
    return i[external];
  }

  abstract boolean go1(int from, int field);

  abstract boolean go2_human(int from, int field);
  void go2() {
    level = 0;
    for (int i = 6; i < 48; i++) {
      bestboard[i] = board[i];
    }
    turn();
    for (int i = 6; i < 48; i++) {
      board[i] = bestboard[i];
    }
  }

  boolean checkCapture1() {
    for (int i = 6; i < 48; i++) {
      if (checkCapture1(i)) {
        return true;
      }
    }
    return false;
  }

  abstract int type();
  abstract boolean checkCapture2();

  abstract boolean checkCapture1(int i);

  abstract void kingMove2(int from, IntGrave resMax);

  abstract boolean manCapture1(int from, int direction, BoolGrave capture);
  abstract boolean kingCapture1(int from, int direction, BoolGrave capture);

  abstract boolean manCapture2(int from, int direction, BoolGrave capture);
  abstract boolean kingCapture2(int from, int direction, BoolGrave capture);
  // for 2 player game
  abstract boolean manCapture2(int from, IntGrave resMax);
  abstract boolean kingCapture2(int from, int direction, IntGrave resMax);
//internals

  int item(int i) {
    return board[internal(i)];
  }

  int turn() {
    IntGrave resMax = new IntGrave((level - levelmax) * 10);
    level++;

    if (checkCapture2()) {
      for (int i = 6; i < 48; i++) {
        switch (board[i]) {
          case MAN2:
            manCapture2(i, resMax);
            break;
          case KING2:
            kingCapture2(i, UL, resMax);
            kingCapture2(i, UR, resMax);
            kingCapture2(i, DL, resMax);
            kingCapture2(i, DR, resMax);
        }
      }

    } else if (checkMove2()) {
      for (int i = 6; i < 48; i++) {
        switch (board[i]) {
          case MAN2:
            if (board[i + 5] == FREE) {	// down left
              board[i] = FREE;
              if (i > 38) {
                board[i + 5] = KING2;
              } else {
                board[i + 5] = MAN2;
              }
              turn(resMax);
              board[i + 5] = FREE;
              board[i] = MAN2;
            }
            if (board[i + 6] == FREE) {	// down right
              board[i] = FREE;
              if (i > 38) {
                board[i + 6] = KING2;
              } else {
                board[i + 6] = MAN2;
              }
              turn(resMax);
              board[i + 6] = FREE;
              board[i] = MAN2;
            }
            break;
          case KING2:
            kingMove2(i, resMax);
            break;
        }
      }
   }

    level--;
    return resMax.get();
  }

  void turn(IntGrave resMax) {
    turn(resMax, false);
  }

  void turn(IntGrave resMax, boolean capture) {
    if (level < levelmax) {
      boolean f12, f13, f14, f17, f18, f19, f23, f24, f25;
      boolean f28, f29, f30, f34, f35, f36, f39, f40, f41;
      f12 = f13 = f14 = f17 = f18 = f19 = f23 = f24 = f25 = false;//for gcc
      f28 = f29 = f30 = f34 = f35 = f36 = f39 = f40 = f41 = false;//for gcc

      if (capture) {
        if (board[12] == NONE) {
          f12 = true;
          board[12] = FREE;
        } else {
          f12 = false;
        }
        if (board[13] == NONE) {
          f13 = true;
          board[13] = FREE;
        } else {
          f13 = false;
        }
        if (board[14] == NONE) {
          f14 = true;
          board[14] = FREE;
        } else {
          f14 = false;
        }
        if (board[17] == NONE) {
          f17 = true;
          board[17] = FREE;
        } else {
          f17 = false;
        }
        if (board[18] == NONE) {
          f18 = true;
          board[18] = FREE;
        } else {
          f18 = false;
        }
        if (board[19] == NONE) {
          f19 = true;
          board[19] = FREE;
        } else {
          f19 = false;
        }
        if (board[23] == NONE) {
          f23 = true;
          board[23] = FREE;
        } else {
          f23 = false;
        }
        if (board[24] == NONE) {
          f24 = true;
          board[24] = FREE;
        } else {
          f24 = false;
        }
        if (board[25] == NONE) {
          f25 = true;
          board[25] = FREE;
        } else {
          f25 = false;
        }
        if (board[28] == NONE) {
          f28 = true;
          board[28] = FREE;
        } else {
          f28 = false;
        }
        if (board[29] == NONE) {
          f29 = true;
          board[29] = FREE;
        } else {
          f29 = false;
        }
        if (board[30] == NONE) {
          f30 = true;
          board[30] = FREE;
        } else {
          f30 = false;
        }
        if (board[34] == NONE) {
          f34 = true;
          board[34] = FREE;
        } else {
          f34 = false;
        }
        if (board[35] == NONE) {
          f35 = true;
          board[35] = FREE;
        } else {
          f35 = false;
        }
        if (board[36] == NONE) {
          f36 = true;
          board[36] = FREE;
        } else {
          f36 = false;
        }
        if (board[39] == NONE) {
          f39 = true;
          board[39] = FREE;
        } else {
          f39 = false;
        }
        if (board[40] == NONE) {
          f40 = true;
          board[40] = FREE;
        } else {
          f40 = false;
        }
        if (board[41] == NONE) {
          f41 = true;
          board[41] = FREE;
        } else {
          f41 = false;
        }
      }

      int b6 = board[6];
      int b7 = board[7];
      int b8 = board[8];
      int b9 = board[9];
      int b11 = board[11];
      int b12 = board[12];
      int b13 = board[13];
      int b14 = board[14];
      int b17 = board[17];
      int b18 = board[18];
      int b19 = board[19];
      int b20 = board[20];
      int b22 = board[22];
      int b23 = board[23];
      int b24 = board[24];
      int b25 = board[25];
      int b28 = board[28];
      int b29 = board[29];
      int b30 = board[30];
      int b31 = board[31];
      int b33 = board[33];
      int b34 = board[34];
      int b35 = board[35];
      int b36 = board[36];
      int b39 = board[39];
      int b40 = board[40];
      int b41 = board[41];
      int b42 = board[42];
      int b44 = board[44];
      int b45 = board[45];
      int b46 = board[46];
      int b47 = board[47];

      board[6] = FULL - b47;
      board[7] = FULL - b46;
      board[8] = FULL - b45;
      board[9] = FULL - b44;
      board[11] = FULL - b42;
      board[12] = FULL - b41;
      board[13] = FULL - b40;
      board[14] = FULL - b39;
      board[17] = FULL - b36;
      board[18] = FULL - b35;
      board[19] = FULL - b34;
      board[20] = FULL - b33;
      board[22] = FULL - b31;
      board[23] = FULL - b30;
      board[24] = FULL - b29;
      board[25] = FULL - b28;
      board[28] = FULL - b25;
      board[29] = FULL - b24;
      board[30] = FULL - b23;
      board[31] = FULL - b22;
      board[33] = FULL - b20;
      board[34] = FULL - b19;
      board[35] = FULL - b18;
      board[36] = FULL - b17;
      board[39] = FULL - b14;
      board[40] = FULL - b13;
      board[41] = FULL - b12;
      board[42] = FULL - b11;
      board[44] = FULL - b9;
      board[45] = FULL - b8;
      board[46] = FULL - b7;
      board[47] = FULL - b6;

      int res = -turn();

      board[6] = b6;
      board[7] = b7;
      board[8] = b8;
      board[9] = b9;
      board[11] = b11;
      board[12] = b12;
      board[13] = b13;
      board[14] = b14;
      board[17] = b17;
      board[18] = b18;
      board[19] = b19;
      board[20] = b20;
      board[22] = b22;
      board[23] = b23;
      board[24] = b24;
      board[25] = b25;
      board[28] = b28;
      board[29] = b29;
      board[30] = b30;
      board[31] = b31;
      board[33] = b33;
      board[34] = b34;
      board[35] = b35;
      board[36] = b36;
      board[39] = b39;
      board[40] = b40;
      board[41] = b41;
      board[42] = b42;
      board[44] = b44;
      board[45] = b45;
      board[46] = b46;
      board[47] = b47;

      if (res > resMax.get()) {
        resMax.set(res);
        if (level == 1) {
          for (int i = 6; i < 48; i++) {
            bestboard[i] = board[i];
          }
          bestcounter = 1;
        }
      } else if (res == resMax.get() && level == 1) {
        bestcounter++;
        if (rand.nextInt()%bestcounter == 0) {
          for (int i = 6; i < 48; i++) {
            bestboard[i] = board[i];
          }
        }
      }

      if (capture) {
        if (f12) {
          board[12] = NONE;
        }
        if (f13) {
          board[13] = NONE;
        }
        if (f14) {
          board[14] = NONE;
        }
        if (f17) {
          board[17] = NONE;
        }
        if (f18) {
          board[18] = NONE;
        }
        if (f19) {
          board[19] = NONE;
        }
        if (f23) {
          board[23] = NONE;
        }
        if (f24) {
          board[24] = NONE;
        }
        if (f25) {
          board[25] = NONE;
        }
        if (f28) {
          board[28] = NONE;
        }
        if (f29) {
          board[29] = NONE;
        }
        if (f30) {
          board[30] = NONE;
        }
        if (f34) {
          board[34] = NONE;
        }
        if (f35) {
          board[35] = NONE;
        }
        if (f36) {
          board[36] = NONE;
        }
        if (f39) {
          board[39] = NONE;
        }
        if (f40) {
          board[40] = NONE;
        }
        if (f41) {
          board[41] = NONE;
        }
      }
    } else if (resMax.get() < 0) {
      resMax.set(0);
    }
  }

  boolean checkMove2() {
    for (int i = 6; i < 48; i++) {
      switch (board[i]) {
        case MAN2:
          if (board[i + 5] == FREE) {
            return true;
          }
          if (board[i + 6] == FREE) {
            return true;
          }
          break;
        case KING2:
          if (board[i - 6] == FREE) {
            return true;
          }
          if (board[i - 5] == FREE) {
            return true;
          }
          if (board[i + 5] == FREE) {
            return true;
          }
          if (board[i + 6] == FREE) {
            return true;
          }
      }
    }
    return false;
  }
  boolean checkMove1(){
    for(int i=6;i<48;i++)
	    if(checkMove1(i))
	      return true;
    return false;
  }
  boolean checkMove1(int i)
{
    switch(board[i]) {
    case MAN1:
	if(board[i-6]==FREE) return true;
	if(board[i-5]==FREE) return true;
	break;
    case KING1:
	if(board[i-6]==FREE) return true;
	if(board[i-5]==FREE) return true;
	if(board[i+5]==FREE) return true;
	if(board[i+6]==FREE) return true;
    }
    return false;
}

  int level;        // Current level
  int levelmax;     // Maximum level
  int to;
  int[] board = new int[54];
  int[] bestboard = new int[54];
  int bestcounter;
  boolean game_over;
  int  winner;

  java.util.Random rand;

  boolean check_win(boolean human_move){
	  if(game_over) return true;
	  boolean player_can = checkMove1() || checkCapture1();
	  boolean opp_can =    checkMove2() || checkCapture2();
	  // AI win
	  if(human_move && !player_can && opp_can){
	    winner=1;
	    game_over = true;
	    return game_over;
	  }
	  // Human win
	  if(!human_move && player_can && !opp_can){
	    winner=2;
	 	  game_over = true;
      return game_over;
	  }
	  // Nobody cam move
	  if(!player_can && !opp_can) {
	    winner=3;
		  game_over = true;
      return game_over;
    }
	  game_over = false;
	  return game_over;
  };

  GameInternal(boolean white,int lvl) {
    rand = new java.util.Random();
    newGame(white, lvl);
  }
  void newGame(boolean white,int lvl){
    for (int i = 0; i < 54; i++) {
      board[i] = NONE;
    }

    for (int i = 0; i < 12; i++) {
      board[internal(i)] = MAN2;
    }
    for (int i = 12; i < 20; i++) {
      board[internal(i)] = FREE;
    }
    for (int i = 20; i < 32; i++) {
      board[internal(i)] = MAN1;
    }

    levelmax = lvl;
    game_over=false;
    if(!white)go2();
  }
};

